/*
 * Sixth 3D engine. Author: Svjatoslav Agejenko. 
 * This project is released under Creative Commons Zero (CC0) license.
 */
package eu.svjatoslav.sixth.e3d.geometry;

import static java.lang.Math.*;

/**
 * Used to represent point in a 3D space or vector.
 */

public class Point3D implements Cloneable {

    public double x, y, z;

    public Point3D() {
    }

    public Point3D(final double x, final double y, final double z) {
        this.x = x;
        this.y = y;
        this.z = z;
    }

    public Point3D(final float x, final float y, final float z) {
        this.x = x;
        this.y = y;
        this.z = z;
    }

    public Point3D(final int x, final int y, final int z) {
        this.x = x;
        this.y = y;
        this.z = z;
    }

    /**
     * Creates new current point by cloning coordinates from parent point.
     */
    public Point3D(final Point3D parent) {
        x = parent.x;
        y = parent.y;
        z = parent.z;
    }

    /**
     * Add other point to current point.
     *
     * @return current point.
     */
    public Point3D add(final Point3D otherPoint) {
        x += otherPoint.x;
        y += otherPoint.y;
        z += otherPoint.z;
        return this;
    }

    public Point3D clone() {
        return new Point3D(this);
    }

    public Point3D clone(final Point3D parent) {
        x = parent.x;
        y = parent.y;
        z = parent.z;
        return this;
    }

    public Point3D computeMiddlePoint(final Point3D p1, final Point3D p2) {
        x = (p1.x + p2.x) / 2d;
        y = (p1.y + p2.y) / 2d;
        z = (p1.z + p2.z) / 2d;
        return this;
    }

    public boolean isZero() {
        return (x == 0) && (y == 0) && (z == 0);
    }

    public double getAngleXZ(final Point3D anotherPoint) {
        return Math.atan2(x - anotherPoint.x, z - anotherPoint.z);
    }

    public double getAngleYZ(final Point3D anotherPoint) {
        return Math.atan2(y - anotherPoint.y, z - anotherPoint.z);
    }

    public double getAngleXY(final Point3D anotherPoint) {
        return Math.atan2(x - anotherPoint.x, y - anotherPoint.y);
    }

    /**
     * Compute distance to another point.
     */
    public double getDistanceTo(final Point3D anotherPoint) {
        final double xDelta = x - anotherPoint.x;
        final double yDelta = y - anotherPoint.y;
        final double zDelta = z - anotherPoint.z;

        return sqrt(((xDelta * xDelta) + (yDelta * yDelta) + (zDelta * zDelta)));
    }

    public double getVectorLength() {
        return sqrt(((x * x) + (y * y) + (z * z)));
    }

    public Point3D invert() {
        x = -x;
        y = -y;
        z = -z;
        return this;
    }

    public void rotate(final Point3D center, final double angleXZ,
                       final double angleYZ) {
        final double s1 = sin(angleXZ);
        final double c1 = cos(angleXZ);

        final double s2 = sin(angleYZ);
        final double c2 = cos(angleYZ);

        x -= center.x;
        y -= center.y;
        z -= center.z;

        final double y1 = (z * s2) + (y * c2);
        final double z1 = (z * c2) - (y * s2);

        final double x1 = (z1 * s1) + (x * c1);
        final double z2 = (z1 * c1) - (x * s1);

        x = x1 + center.x;
        y = y1 + center.y;
        z = z2 + center.z;
    }

    public void roundToInteger() {
        x = (int) x;
        y = (int) y;
        z = (int) z;
    }

    public Point3D scaleDown(final double factor) {
        x /= factor;
        y /= factor;
        z /= factor;
        return this;
    }

    public Point3D scaleUp(final double factor) {
        x *= factor;
        y *= factor;
        z *= factor;
        return this;
    }

    public void setValues(final double x, final double y, final double z) {
        this.x = x;
        this.y = y;
        this.z = z;
    }

    /**
     * Subtract other point from current point.
     *
     * @return current point.
     */
    public Point3D subtract(final Point3D otherPoint) {
        x -= otherPoint.x;
        y -= otherPoint.y;
        z -= otherPoint.z;
        return this;
    }

    @Override
    public String toString() {
        return "x:" + x + " y:" + y + " z:" + z;
    }

    /**
     * @return current point.
     */
    public Point3D translateX(final double xIncrement) {
        x += xIncrement;
        return this;
    }

    /**
     * @return current point.
     */
    public Point3D translateY(final double yIncrement) {
        y += yIncrement;
        return this;
    }

    /**
     * @return current point.
     */
    public Point3D translateZ(final double zIncrement) {
        z += zIncrement;
        return this;
    }

    /**
     * Here we assume that Z coordinate is distance to the viewer.
     * If Z is positive, then point is in front of the viewer, and therefore it is visible.
     *
     * @return point visibility status.
     */
    public boolean isVisible() {
        return z > 0;
    }

    /**
     * Resets point to 0 coordinate in X, Y and Z axis.
     *
     * @return current point.
     */
    public Point3D zero() {
        x = 0;
        y = 0;
        z = 0;
        return this;
    }

}
