/*
 * Sixth 3D engine. Author: Svjatoslav Agejenko. 
 * This project is released under Creative Commons Zero (CC0) license.
 */
package eu.svjatoslav.sixth.e3d.renderer.octree.raytracer;

import eu.svjatoslav.sixth.e3d.geometry.Point3D;
import eu.svjatoslav.sixth.e3d.gui.Avatar;
import eu.svjatoslav.sixth.e3d.math.Transform;
import eu.svjatoslav.sixth.e3d.renderer.raster.Color;
import eu.svjatoslav.sixth.e3d.renderer.raster.shapes.basic.line.LineAppearance;
import eu.svjatoslav.sixth.e3d.renderer.raster.shapes.composite.TexturedRectangle;
import eu.svjatoslav.sixth.e3d.renderer.raster.texture.Texture;

import javax.imageio.ImageIO;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.net.URL;

public class Camera extends TexturedRectangle {

    public static final int CAMERA_SIZE = 100;
    public static final int IMAGE_SIZE = 500;
    private final CameraView cameraView;
    private Point3D camCenter;

    public Camera(final Avatar avatar, final double zoom) {
        super(new Transform(avatar.getLocation().clone()));
        cameraView = new CameraView(avatar, zoom);

        computeCameraCoordinates(avatar);

        addWaitNotification(getTexture());
    }

    private void addWaitNotification(final Texture texture) {
        // add hourglass icon
        try {
            final BufferedImage sprite = getSprite("eu/svjatoslav/sixth/e3d/examples/hourglass.png");
            texture.graphics.drawImage(sprite, IMAGE_SIZE / 2,
                    (IMAGE_SIZE / 2) - 30, null);
        } catch (final Exception ignored) {
        }

        // add "Please wait..." message
        texture.graphics.setColor(java.awt.Color.WHITE);
        texture.graphics.setFont(new Font("Monospaced", Font.PLAIN, 10));
        texture.graphics.drawString("Please wait...", (IMAGE_SIZE / 2) - 20,
                (IMAGE_SIZE / 2) + 30);
    }

    private void computeCameraCoordinates(final Avatar avatar) {
        initialize(CAMERA_SIZE, CAMERA_SIZE, IMAGE_SIZE, IMAGE_SIZE, 3);

        camCenter = new Point3D();

        topLeft.setValues(camCenter.x, camCenter.y, camCenter.z + CAMERA_SIZE);

        topRight.clone(topLeft);
        bottomLeft.clone(topLeft);
        bottomRight.clone(topLeft);

        final float viewAngle = (float) .6;

        topLeft.rotate(camCenter, -viewAngle, -viewAngle);
        topRight.rotate(camCenter, viewAngle, -viewAngle);
        bottomLeft.rotate(camCenter, -viewAngle, viewAngle);
        bottomRight.rotate(camCenter, viewAngle, viewAngle);

        topLeft.rotate(camCenter, -avatar.getAngleXZ(), -avatar.getAngleYZ());
        topRight.rotate(camCenter, -avatar.getAngleXZ(), -avatar.getAngleYZ());
        bottomLeft.rotate(camCenter, -avatar.getAngleXZ(), -avatar.getAngleYZ());
        bottomRight.rotate(camCenter, -avatar.getAngleXZ(), -avatar.getAngleYZ());

        final Color cameraColor = new Color(255, 255, 0, 255);
        final LineAppearance appearance = new LineAppearance(2, cameraColor);

        // addShape(appearance.getLine(camCenter, upLeft));
        // addShape(appearance.getLine(camCenter, upRight));
        // addShape(appearance.getLine(camCenter, downLeft));
        // addShape(appearance.getLine(camCenter, downRight));

        addShape(appearance.getLine(topLeft, topRight));
        addShape(appearance.getLine(bottomLeft, bottomRight));
        addShape(appearance.getLine(topLeft, bottomLeft));
        addShape(appearance.getLine(topRight, bottomRight));

    }

    public CameraView getCameraView() {
        return cameraView;
    }

    public BufferedImage getSprite(final String ref) throws IOException {
        final URL url = this.getClass().getClassLoader().getResource(ref);
        return ImageIO.read(url);
    }

}
