/*
 * Sixth 3D engine. Author: Svjatoslav Agejenko. 
 * This project is released under Creative Commons Zero (CC0) license.
 */
package eu.svjatoslav.sixth.e3d.renderer.raster.shapes.composite;

import eu.svjatoslav.sixth.e3d.geometry.Point2D;
import eu.svjatoslav.sixth.e3d.geometry.Point3D;
import eu.svjatoslav.sixth.e3d.math.Transform;
import eu.svjatoslav.sixth.e3d.renderer.raster.Color;
import eu.svjatoslav.sixth.e3d.renderer.raster.shapes.basic.line.Line;
import eu.svjatoslav.sixth.e3d.renderer.raster.shapes.composite.base.AbstractCompositeShape;
import eu.svjatoslav.sixth.e3d.renderer.raster.shapes.composite.textcanvas.TextCanvas;

import java.util.List;

public class Graph extends AbstractCompositeShape {

    private final double width;
    private final double yMin;
    private final double yMax;
    private final double horizontalStep;
    private final double verticalStep;
    private final Color gridColor;
    private final double lineWidth;
    private final Color plotColor;

    public Graph(final double scale, final List<Point2D> data,
                 final String label, final Point3D location) {
        super(location);

        width = 20;

        yMin = -2;
        yMax = 2;

        horizontalStep = 0.5;
        verticalStep = 0.5;

        gridColor = new Color(100, 100, 250, 100);

        lineWidth = 0.1 * scale;
        plotColor = new Color(255, 0, 0, 100);

        addVerticalLines(scale);
        addXLabels(scale);
        addHorizontalLinesAndLabels(scale);
        plotData(scale, data);

        final Point3D labelLocation = new Point3D(width / 2, yMax + 0.5, 0)
                .scaleUp(scale);

        final TextCanvas labelCanvas = new TextCanvas(new Transform(
                labelLocation), label, Color.WHITE, Color.TRANSPARENT);

        addShape(labelCanvas);
    }

    private void addHorizontalLinesAndLabels(final double scale) {
        for (double y = yMin; y <= yMax; y += verticalStep) {

            final Point3D p1 = new Point3D(0, y, 0).scaleUp(scale);

            final Point3D p2 = new Point3D(width, y, 0).scaleUp(scale);

            final Line line = new Line(p1, p2, gridColor, lineWidth);

            addShape(line);

            final Point3D labelLocation = new Point3D(-0.5, y, 0)
                    .scaleUp(scale);

            final TextCanvas label = new TextCanvas(
                    new Transform(labelLocation), String.valueOf(y),
                    Color.WHITE, Color.TRANSPARENT);

            addShape(label);

        }
    }

    private void addVerticalLines(final double scale) {
        for (double x = 0; x <= width; x += horizontalStep) {

            final Point3D p1 = new Point3D(x, yMin, 0).scaleUp(scale);
            final Point3D p2 = new Point3D(x, yMax, 0).scaleUp(scale);

            final Line line = new Line(p1, p2, gridColor, lineWidth);

            addShape(line);

        }
    }

    private void addXLabels(final double scale) {
        for (double x = 0; x <= width; x += horizontalStep * 2) {
            final Point3D labelLocation = new Point3D(x, yMin - 0.4, 0)
                    .scaleUp(scale);

            final TextCanvas label = new TextCanvas(
                    new Transform(labelLocation), String.valueOf(x),
                    Color.WHITE, Color.TRANSPARENT);

            addShape(label);
        }
    }

    private void plotData(final double scale, final List<Point2D> data) {
        Point3D previousPoint = null;
        for (final Point2D point : data) {

            final Point3D p3d = new Point3D(point.x, point.y, 0).scaleUp(scale);

            if (previousPoint != null) {

                final Line line = new Line(previousPoint, p3d, plotColor,
                        0.4 * scale);

                addShape(line);
            }

            previousPoint = p3d;
        }
    }
}
